local _G = getfenv(0)
local ADDON_NAME, addon = ...

local LibStub = _G.LibStub
local CompactRunes = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)
local L = LibStub("AceLocale-3.0"):GetLocale(addon.addonName)

-- Use BfA+ version to search by name.
local UnitBuff = addon.UnitBuff
local UnitDebuff = addon.UnitDebuff

local ipairs = _G.ipairs
local tostring = _G.tostring
local tonumber = _G.tonumber
local GetTime = _G.GetTime
local GetSpellCooldown = _G.GetSpellCooldown
local GetSpellCharges = _G.GetSpellCharges
local FormatNumber = addon.FormatNumber

-- Locals for spell data.
local SpellIds = addon.SpellIds
local SpellNames = addon.SpellNames
local SpellIcons = addon.SpellIcons
-- Local definition of Bar
local Bar = addon.Bar

local frames = addon.frames

function CompactRunes:CreateOtherFrames()
	frames.boneBar = Bar:Create({
		name = "BoneShieldBar",
		fontAdj = 2,
		initTimer = true,
		functions = {
			GetWidth = function(self)
				return self.db.overrideWidth and self.db.width or addon.GetHalfBarWidth()
			end,
			GetHeight = function(self)
				return self.db.overrideHeight and self.db.height or addon.GetHalfBarHeight()
			end,
			SetPoint = addon.layouts["topbar1-right"],
			SetValuePoint = function(self)
				self.bar.value:SetPoint("CENTER", self.bar, "TOP", 0, 0)
			end,
			UpdateDisplay = function(self)
				if self.active then
					name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
						= UnitBuff("player", SpellNames["Bone Shield"])
					if name then
						local timeLeft = expires - GetTime()
						self.bar.timer = timeLeft
						self.bar.active = true
						if count ~= self.bar.count then
							self.bar.value:SetText(tostring(count))
						end
						self.bar.count = count

						if self.db.progress == "Time" then
							self.bar:SetMinMaxValues(0, duration)
						elseif self.db.progress == "Charges" then
							self.bar:SetMinMaxValues(0, addon.MAX_BONESHIELD_CHARGES)
						else
							self.bar:SetMinMaxValues(0, 1)
							self.bar:SetValue(1)
						end
						self.bar:SetAlpha(1)
						self.bar.value:Show()
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateBoneShield)
					else
						self.bar.active = false
						self.bar.timer = 0
						self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				else
					self.bar:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled and
					_G.IsSpellKnown(SpellIds["Marrowrend"])
			end,
			OnTalentUpdate = function(self)
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					addon:BarDisplayAdd("PlayerAura", self)
					self:UpdateDisplay()
				else
					self.active = false
					addon:BarDisplayRemove("PlayerAura", self)
					self:Hide()
				end
			end,
			UpdateBoneShieldBarMode = function(self)
				if self.db.progress == "Time" then
					self.bar:SetMinMaxValues(0, 1)
					self.bar:SetValue(1)
				elseif self.db.progress == "Charges" then
					self.bar:SetMinMaxValues(0, addon.MAX_BONESHIELD_CHARGES)
					self.bar:SetValue(0)
				elseif self.db.progress == "None" then
					self.bar:SetMinMaxValues(0, 1)
					self.bar:SetValue(1)
				end
			end
		}
	})
	frames.boneBar:OnTalentUpdate()
	frames.boneBar:UpdateBoneShieldBarMode()

	frames.bsBar = Bar:Create({
		name = "BloodShieldBar",
		fontAdj = 2,
		initTimer = true,
		setValueToZero = true,
		updateVisibilityOnLock = true,
		layout = { "bottombar2-full" },
		functions = {
			UpdateDisplay = function(self)
				local name, icon, count, dispelType, duration, expires,
				caster, isStealable, shouldConsolidate, spellId, canApplyAura,
				isBossDebuff, castByPlayer, value1, value2, value3
					= UnitBuff("player", SpellNames["Blood Shield"])
				if name then
					self.bar.timer = expires - GetTime()
					self.bar:SetMinMaxValues(0, duration)
					self.bar.value:SetText(FormatNumber(value3 or 0))
					self.bar.active = true
					self.bar:Show()
					self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerNoCounter)
				else
					self.bar.active = false
					self.bar.timer = 0
					self.bar:SetScript("OnUpdate", nil)
					self.bar:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				local events = { "PlayerAura" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.bsBar:OnTalentUpdate()

	frames.ffBar = Bar:Create({
		name = "FrostFeverBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-left" },
		functions = {
			IsEnabled = function(self)
				return addon.currentSpec == "Frost" and self.db.enabled
			end,
		}
	})

	frames.bpBar = Bar:Create({
		name = "BloodPlagueBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-left" },
		functions = {
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled
			end,
		}
	})

	frames.vpBar = Bar:Create({
		name = "VirulentPlagueBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-left" },
		functions = {
			IsEnabled = function(self)
				return addon.currentSpec == "Unholy" and self.db.enabled
			end,
		}
	})

	frames.fwBar = Bar:Create({
		name = "FesteringWoundBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-right" },
		functions = {
			UpdateDisplay = function(self)
				local name, icon, count, dispelType, duration, expires,
				caster, isStealable, shouldConsolidate, spellId, canApplyAura,
				isBossDebuff, castByPlayer, value1, value2, value3
				= UnitDebuff("target", SpellNames["Festering Wound"], "PLAYER")
				if name then
					self.bar.active = true
					local timeLeft = expires - GetTime()
					self.bar:SetMinMaxValues(0, duration)
					self.bar.timer = timeLeft
					self.bar:Show()
					self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerNoCounter)
					self.bar.value:SetText(tostring(count))
				else
					self.bar.active = false
					self.bar.timer = 0
					self.bar:SetScript("OnUpdate", nil)
					self.bar:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Unholy" and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				local events = { "TargetAura" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})

	frames.ddBar = Bar:Create({
		name = "DeathAndDecayBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "leftbar-full" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local show = false
					name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
					= UnitBuff("player", SpellNames["Death and Decay Buff"])
					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						local bc = self.db.color
						local tc = self.db.textcolor
						self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
						self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
						show = true
					else
						if self.db.showCooldown then
							local start, duration, enabled = GetSpellCooldown(SpellIds[self.spellforcd])
							if duration > addon.runeCD then
								self.bar.timer = start + duration - GetTime()
								local bc = self.db.alt_color
								local tc = self.db.alt_textcolor
								self.bar:SetMinMaxValues(0, duration)
								self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
								self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
								self.bar.active = true
								self.bar:Show()
								self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
								show = true
							end
						end
					end
					if not show then
						self.bar.active = false
						self.bar.timer = 0
						self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				end
			end,
			IsEnabled = function(self)
				return self.db.specs[addon.currentSpec] and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					self.spellforcd = addon.HasActiveTalent("Defile") and "Defile" or "Death and Decay"
					addon:BarDisplayAdd("PlayerAura", self)
					self:UpdateDisplay()
				else
					self.active = false
					addon:BarDisplayRemove("PlayerAura", self)
					self:Hide()
				end
			end
		}
	})
	frames.ddBar:OnTalentUpdate()

	frames.rsBar = Bar:Create({
		name = "RuneStrikeBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar2-full", "topbar2-left" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					if event == "CombatStart" then
						self.bar:SetAlpha(self.db.alpha)
						return
					end
					local current, max, start, duration = GetSpellCharges(SpellIds["Rune Strike"])
					if current and current >= 0 then
						self.bar.value:SetText(tostring(current))
						self.bar:SetMinMaxValues(0, duration)
						if current >= max then
							self.bar:SetValue(duration)
							self.bar.active = false
							self.bar:SetScript("OnUpdate", nil)
							self:Show()
							self.bar:SetAlpha(_G.UnitAffectingCombat("player") and
								self.db.alpha or addon.db.profile.auxBarOOCAlpha)
						else
							local remaining = GetTime() - start
							self.bar:SetValue(remaining)
							self.bar.active = true
							self.bar.duration = duration
							self.bar.timer = remaining
							self:Show()
							self.bar:SetScript("OnUpdate", addon.scripts.onUpdateIncreasingTimerNoCounter)
							self.bar:SetAlpha(self.db.alpha)
						end
					else
						self:Hide()
						self.bar.active = false
						self.bar:SetScript("OnUpdate", nil)
					end
				else
					self:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled and addon.HasActiveTalent("Rune Strike")
			end,
			OnTalentUpdate = function(self)
				local events = { "SpellCooldown", "CombatStart", "CombatEnd" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end,
		}
	})
	frames.rsBar:OnTalentUpdate()

	frames.bbBar = Bar:Create({
		name = "BloodBoilBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar2-full", "topbar2-left" },
		functions = {
			UpdateDisplay = function(self, event)
				if self.active then
					if event == "CombatStart" then
						self.bar:SetAlpha(self.db.alpha)
						return
					end
					local current, max, start, duration = GetSpellCharges(SpellIds["Blood Boil"])
					if current and current >= 0 then
						self.bar.value:SetText(tostring(current))
						self.bar:SetMinMaxValues(0, duration)
						if current >= max then
							self.bar:SetValue(duration)
							self.bar.active = false
							self.bar:SetScript("OnUpdate", nil)
							self:Show()
							self.bar:SetAlpha(_G.UnitAffectingCombat("player") and
								self.db.alpha or addon.db.profile.auxBarOOCAlpha)
						else
							local remaining = GetTime() - start
							self.bar:SetValue(remaining)
							self.bar.active = true
							self.bar.duration = duration
							self.bar.timer = remaining
							self:Show()
							self.bar:SetScript("OnUpdate", addon.scripts.onUpdateIncreasingTimerNoCounter)
							self.bar:SetAlpha(self.db.alpha)
						end
					else
						self:Hide()
						self.bar.active = false
						self.bar:SetScript("OnUpdate", nil)
					end
				else
					self:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				local events = { "SpellCooldown", "CombatStart", "CombatEnd" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.bbBar:OnTalentUpdate()

	frames.rtBar = Bar:Create({
		name = "RuneTapBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar2-full", "topbar2-right" },
		functions = {
			UpdateDisplay = function(self, event)
				if self.active then
					if event == "CombatStart" then
						self.bar:SetAlpha(self.db.alpha)
						return
					end
					local current, max, start, duration = GetSpellCharges(SpellIds["Rune Tap"])
					if current and current >= 0 then
						self.bar.value:SetText(tostring(current))
						self.bar:SetMinMaxValues(0, duration)
						if current >= max then
							self.bar:SetValue(duration)
							self.bar.active = false
							self.bar:SetScript("OnUpdate", nil)
							self:Show()
							self.bar:SetAlpha(_G.UnitAffectingCombat("player") and
								self.db.alpha or addon.db.profile.auxBarOOCAlpha)
						else
							local remaining = GetTime() - start
							self.bar:SetValue(remaining)
							self.bar.active = true
							self.bar.duration = duration
							self.bar.timer = remaining
							self:Show()
							self.bar:SetScript("OnUpdate", addon.scripts.onUpdateIncreasingTimerNoCounter)
							self.bar:SetAlpha(self.db.alpha)
						end
					else
						self:Hide()
						self.bar.active = false
						self.bar:SetScript("OnUpdate", nil)
					end
				else
					self:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				local events = { "SpellCooldown", "CombatStart", "CombatEnd" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.rtBar:OnTalentUpdate()


	frames.vbBar = Bar:Create({
		name = "VampiricBloodBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "rightbar-full" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local show = false
					name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
					= UnitBuff("player", SpellNames["Vampiric Blood"])
					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						local bc = self.db.color
						local tc = self.db.textcolor
						self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
						self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
						show = true
					else
						if self.db.showCooldown then
							local start, duration, enabled = GetSpellCooldown(SpellIds["Vampiric Blood"])
							if duration > 2 then
								self.bar.timer = start + duration - GetTime()
								local bc = self.db.alt_color
								local tc = self.db.alt_textcolor
								self.bar:SetMinMaxValues(0, duration)
								self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
								self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
								self.bar.active = true
								self.bar:Show()
								self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
								show = true
							end
						end
					end
					if not show then
						self.bar.active = false
						self.bar.timer = 0
						self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				local events = { "SpellCooldown", "PlayerAura" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.vbBar:OnTalentUpdate()

	frames.dtBar = Bar:Create({
		name = "DarkTransformationBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar2-full", "topbar2-left" },
		functions = {
			UpdateDisplay = function(self)
				local show = false
				if self.active then
					name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
					= UnitBuff("pet", SpellNames["Dark Transformation"], "PLAYER")

					if name then
						self.bar.dt = true
						self.bar.timer = expires - GetTime()
						local bc, tc
						if self.bar.dt then
							bc = self.db.color
							tc = self.db.textcolor
						else
							bc = self.db.alt_color
							tc = self.db.alt_textcolor
						end
						self.bar:SetMinMaxValues(0, duration)
						self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
						self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
						show = true
					else
						-- If set, show the cooldown bar.
						self.bar.dt = false
						if self.showCooldown or self.db.showReady then
							local start, duration, enabled = GetSpellCooldown(SpellIds["Dark Transformation"])
							if duration > 2 then
                local remaining = start + duration - GetTime()
                if self.showReady then
                  if self.bar.spellTimer and not self.bar.spellTimer:IsCancelled() then
                    self.bar.spellTimer:Cancel()
                  end
                  self.bar.spellTimer = C_Timer.NewTimer(remaining, function()
                    self.bar.spellTimer = nil
                    self:UpdateDisplay()
                  end)
                end
                if self.showCooldown then
  								self.bar.timer = remaining
  								local bc = self.db.alt_color
  								local tc = self.db.alt_textcolor
  								self.bar:SetMinMaxValues(0, duration)
  								self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
  								self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
  								self.bar.active = true
  								self.bar:Show()
  								self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounterExpire)
  								show = true
                end
              elseif self.db.showReady and duration < 2 then
                self.bar:SetScript("OnUpdate", nil)
                self.bar.timer = 0
                local bc = self.db.tertiary_color
								self.bar:SetMinMaxValues(0, 1)
                self.bar:SetValue(1)
								self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
                self.bar.value:SetTextColor(1, 1, 1, 0)
								self.bar.active = true
								self.bar:Show()
								show = true
							end
						end
					end
				end
				if not show then
					self.bar.active = false
					self.bar.timer = 0
					self.bar:SetScript("OnUpdate", nil)
					self.bar:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Unholy" and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				local events = { "SpellCooldown", "PetAura", "CombatStart", "CombatEnd" }
				self.showCooldown = self.db.showCooldown == "Always"
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end,
      OnTimerExpire = function(self)
        C_Timer.After(0.5, function() self:UpdateDisplay() end)
      end,
		}
	})
	frames.dtBar:OnTalentUpdate()

	frames.epBar = Bar:Create({
		name = "EpidemicBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar2-right", "rightbar-full" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					if event == "CombatStart" then
						self.bar:SetAlpha(self.db.alpha)
						return
					end
					local current, max, start, duration = GetSpellCharges(SpellIds["Epidemic"])
					if current and current >= 0 then
						self.bar.value:SetText(tostring(current))
						self.bar:SetMinMaxValues(0, duration)
						if current >= max then
							self.bar:SetValue(duration)
							self.bar.active = false
							self.bar:SetScript("OnUpdate", nil)
							self:Show()
							self.bar:SetAlpha(_G.UnitAffectingCombat("player") and
								self.db.alpha or addon.db.profile.auxBarOOCAlpha)
						else
							local remaining = GetTime() - start
							self.bar:SetValue(remaining)
							self.bar.active = true
							self.bar.duration = duration
							self.bar.timer = remaining
							self:Show()
							self.bar:SetScript("OnUpdate", addon.scripts.onUpdateIncreasingTimerNoCounter)
							self.bar:SetAlpha(self.db.alpha)
						end
					else
						self:Hide()
						self.bar.active = false
						self.bar:SetScript("OnUpdate", nil)
					end
				else
					self:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Unholy" and self.db.enabled and addon.HasActiveTalent("Epidemic")
			end,
			OnTalentUpdate = function(self)
				local events = { "SpellCooldown", "CombatStart", "CombatEnd" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.epBar:OnTalentUpdate()

	frames.srBar = Bar:Create({
		name = "SoulReaperBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar2-right", "rightbar-full" },
		secondaryFontAdj = 0,
		functions = {
			UpdateDisplay = function(self)
				local show = false
				if self.active then
					local name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
					= UnitDebuff("target", SpellNames["Soul Reaper"], "PLAYER")

					if name then
						self.bar.timer = expires - GetTime()
						local bc = self.db.color
						local tc = self.db.textcolor
						self.bar:SetMinMaxValues(0, duration)
						self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
						self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
						show = true
					elseif self.db.showCooldown then
						show = true
					end
				end
				if not show then
					self.bar.active = false
					self.bar.timer = 0
					self.bar:SetScript("OnUpdate", nil)
					self.bar:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Unholy" and self.db.enabled and addon.HasActiveTalent("Soul Reaper")
			end,
			OnTalentUpdate = function(self)
        local events = { "TargetAura" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
          for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
          for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.srBar:OnTalentUpdate()

	frames.iaBar = Bar:Create({
		name = "InexorableAssaultBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-full", "topbar1-right" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
						= UnitBuff("player", SpellNames["Inexorable Assault"])

					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						self.bar.active = true
						self.bar.value:SetText(tostring(count))
						self.bar:Show()
						--self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerNoCounter)
					else
						self.bar.active = false
						self.bar.timer = 0
						--self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				else
					self:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Frost" and self.db.enabled and addon.HasActiveTalent("Inexorable Assault")
			end,
			OnTalentUpdate = function(self)
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					addon:BarDisplayAdd("PlayerAura", self)
					self:UpdateDisplay()
				else
					addon:BarDisplayRemove("PlayerAura", self)
					self.active = false
					self:Hide()
				end
			end
		}
	})
	frames.iaBar:OnTalentUpdate()

	frames.itBar = Bar:Create({
		name = "IcyTalonsBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-full", "topbar1-right" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
						= UnitBuff("player", SpellNames["Icy Talons"])

					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						self.bar.active = true
						self.bar.value:SetText(tostring(count))
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerNoCounter)
					else
						self.bar.active = false
						self.bar.timer = 0
						self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				else
					self:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Frost" and self.db.enabled and addon.HasActiveTalent("Icy Talons")
			end,
			OnTalentUpdate = function(self)
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					addon:BarDisplayAdd("PlayerAura", self)
					self:UpdateDisplay()
				else
					addon:BarDisplayRemove("PlayerAura", self)
					self.active = false
					self:Hide()
				end
			end
		}
	})
	frames.itBar:OnTalentUpdate()

	frames.chBar = Bar:Create({
		name = "ColdHeartBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-full", "topbar1-right" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
						= UnitBuff("player", SpellNames["Cold Heart Buff"])

					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						self.bar.active = true
						self.bar.value:SetText(tostring(count))
						self.bar:Show()
						--self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerNoCounter)
					else
						self.bar.active = false
						self.bar.timer = 0
						--self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				else
					self:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Frost" and self.db.enabled and addon.HasActiveTalent("Cold Heart")
			end,
			OnTalentUpdate = function(self)
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					addon:BarDisplayAdd("PlayerAura", self)
					self:UpdateDisplay()
				else
					addon:BarDisplayRemove("PlayerAura", self)
					self.active = false
					self:Hide()
				end
			end
		}
	})
	frames.chBar:OnTalentUpdate()

	frames.riBar = Bar:Create({
		name = "RazoriceBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-full", "topbar1-right" },
		functions = {
			-- GetWidth = function(self)
			-- 	return self.db.overrideWidth and self.db.width or addon.GetHalfBarWidth()
			-- end,
			-- GetHeight = function(self)
			-- 	return self.db.overrideHeight and self.db.height or addon.GetHalfBarHeight()
			-- end,
			-- SetPoint = addon.layouts["topbar1-right"],
			-- SetValuePoint = function(self)
			-- 	self.bar.value:SetPoint("CENTER", self.bar, "TOP", 0, 0)
			-- end,
			UpdateDisplay = function(self)
				if self.active then
					local name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
					= UnitDebuff("target", SpellNames["Razorice"], "PLAYER")

					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						self.bar.value:SetText(tostring(count))
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerNoCounter)
					else
						self.bar.active = false
						self.bar.timer = 0
						self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Frost" and self.db.enabled and
					addon.HasActiveTalent("Shattering Strikes")
			end,
			OnTalentUpdate = function(self)
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					addon:BarDisplayAdd("TargetAura", self)
					self:UpdateDisplay()
				else
					self.active = false
					addon:BarDisplayRemove("TargetAura", self)
					self:Hide()
				end
			end
		}
	})
	frames.riBar:OnTalentUpdate()

	frames.rwBar = Bar:Create({
		name = "RemorselessWinterBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar2-full", "topbar2-left" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
					= UnitBuff("player", SpellNames["Remorseless Winter"])

					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						--self.bar.value:SetText(tostring(count))
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
					else
						self.bar.active = false
						self.bar.timer = 0
						self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Frost" and self.db.enabled and addon.HasActiveTalent("Gathering Storm")
			end,
			OnTalentUpdate = function(self)
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					addon:BarDisplayAdd("PlayerAura", self)
					self:UpdateDisplay()
				else
					self.active = false
					addon:BarDisplayRemove("PlayerAura", self)
					self:Hide()
				end
			end
		}
	})
	frames.rwBar:OnTalentUpdate()
end
