local _G = getfenv(0)
local ADDON_NAME, addon = ...

local CompactRunes = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)
local L = _G.LibStub("AceLocale-3.0"):GetLocale(addon.addonName, true)

local UnitBuff = addon.UnitBuff
local UnitDebuff = addon.UnitDebuff
local GetSpellCooldown = _G.GetSpellCooldown
local SpellIds = addon.SpellIds
local SpellNames = addon.SpellNames
local frames = addon.frames
local CombatLogGetCurrentEventInfo = _G.CombatLogGetCurrentEventInfo

function addon.SetupSpellAuras()
	addon.StandardDiseases = {
		["Blood"] = {
			[SpellNames["Blood Plague"]] = frames.bpBar,
		},
		["Frost"] = {
			[SpellNames["Frost Fever"]] = frames.ffBar,
		},
		["Unholy"] = {
			[SpellNames["Virulent Plague"]] = frames.vpBar,
		},
	}
	addon.petBuffs = {
		SpellNames["Dark Transformation"],
	}
end

function addon.GetRuneOrderMap()
	return {
		["Right"] = L["Right"],
		["Down"] = L["Down"],
		["Left"] = L["Left"],
		["Up"] = L["Up"],
	}
end

function addon.NeedsGhoul()
	local start, duration, enabled = GetSpellCooldown(SpellIds["Raise Dead"])
	return not GetTotemInfo(1) and duration <= 0
end

function CompactRunes:CheckSpells()
	self:UpdateSoulReaperEnabled()
end
  
-- If procs settings changed, update any necessary settings.
function CompactRunes:OnProcsChanged()
	self:UpdateSoulReaperEnabled()
end

function CompactRunes:GetMeleeRangeSpells()
	local SpellsBySpec = {
		["Blood"] = "Death Strike",
		["Frost"] = "Frost Strike",
		["Unholy"] = "Scourge Strike",
	}

	return {
		SpellsBySpec[addon.currentSpec] or "",
		"Death Strike",
		"Rune Strike",
		"Rune Strike (starter)"
	}
end

-- Soul Reaper

function CheckSoulReaper()
	CompactRunes:CheckSoulReaper()
end
  
function CompactRunes:UpdateSoulReaperEnabled()
	local enabled = addon.db.profile.enabled[addon.currentSpec]
  
	addon.soulReaperEnabled = addon.currentSpec == "Unholy" and
	  addon.IsSpellKnown("Soul Reaper") and
	  enabled and enabled["Soul Reaper"]
	addon.soulReaperPercent = addon.db.profile.soulReaperPercentEnable and
	  addon.db.profile.soulReaperPercent or addon.SOUL_REAPER_PERCENT
  
	if addon.soulReaperEnabled then
	  addon:RegisterCallback("TargetHealth", "SoulReaper", CheckSoulReaper)
	else
	  addon:UnregisterCallback("TargetHealth", "SoulReaper")
	end
end
 
function CompactRunes:CheckSoulReaper(update)
	-- If below percent, known, and not on CD, show proc.
	if not addon.soulReaperEnabled then return end
  
	local changed = false
	local spellId = SpellIds["Soul Reaper"]
	local start, duration, enabled = GetSpellCooldown(spellId)
	local ready = (duration < 2)
	local usable = _G.IsUsableSpell(spellId)
	local show = (ready and usable and
	  addon.targetPercent <= addon.soulReaperPercent) and 1 or nil
  
	if show ~= self.currentProcs["Soul Reaper"] then
	  self.currentProcs["Soul Reaper"] = show
	  changed = true
	end
  
	if changed and update ~= false then
	  self:UpdateProcIcon()
	end
  
	return changed
  end

function CompactRunes:OnUnitAura(event, unit)
	local name, icon, count, dispelType, duration, expires,
		caster, isStealable, shouldConsolidate, spellId, canApplyAura,
		isBossDebuff, castByPlayer, value1, value2, value3

	if unit == "target" then
		if addon.currentSpec == "Unholy" then
			-- TODO: Function call taking spell name, bar, possibly the onUpdate.
			name, icon, count, dispelType, duration, expires,
			caster, isStealable, shouldConsolidate, spellId, canApplyAura,
			isBossDebuff, castByPlayer, value1, value2, value3
			= UnitDebuff("target", SpellNames["Festering Wound"], "PLAYER")
			local bar = frames.fwBar
			local fwProc = (count == nil or count < 5) and 1 or nil
			local changed = fwProc ~= self.currentProcs["Festering Wound < 5"]
			if changed then
				self.currentProcs["Festering Wound < 5"] = fwProc
				self:UpdateProcIcon()
			end
			-- if name and bar.db.enabled then
			-- 	bar.bar.active = true
			-- 	local timeLeft = expires - GetTime()
			-- 	bar.bar:SetMinMaxValues(0, duration)
			-- 	bar.bar.timer = timeLeft
			-- 	bar.bar:Show()
			-- 	bar.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerNoCounter)
			-- 	bar.bar.value:SetText(tostring(count))
			-- else
			-- 	bar.bar.active = false
			-- 	bar.bar.timer = 0
			-- 	bar.bar:SetScript("OnUpdate", nil)
			-- 	bar.bar:Hide()
			-- end
		end
	elseif unit == "player" then
		if self.db.profile.procs then
			if addon.currentSpec == "Blood" then
				local low = self.db.profile.boneShieldLow or addon.BONE_SHIELD_LOW
				name, icon, count, dispelType, duration, expires,
				caster, isStealable, shouldConsolidate, spellId, canApplyAura,
				isBossDebuff, castByPlayer, value1, value2, value3
				= UnitBuff("player", SpellNames["Bone Shield"], "PLAYER")
				local bsProc = (count == nil or count <= low) and (count or 1) or nil
				local changed = bsProc ~= self.currentProcs["Bone Shield <= X"]
				if changed then
					self.currentProcs["Bone Shield <= X"] = bsProc
				end
			end
		end
	end
end

function CompactRunes:COMBAT_LOG_EVENT_UNFILTERED(...)
	local event = "COMBAT_LOG_EVENT_UNFILTERED"
	local timestamp, eventtype, hideCaster,
	srcGUID, srcName, srcFlags, srcRaidFlags,
	destGUID, destName, destFlags, destRaidFlags,
	param9, param10, param11, param12, param13, param14,
	param15, param16, param17, param18, param19, param20 = CombatLogGetCurrentEventInfo()

	if destGUID == addon.playerGUID then
		if eventtype == "SPELL_AURA_APPLIED" then
			if param9 == SpellIds["Runic Corruption"] then
				self.LastFightStats.rcProcs = self.LastFightStats.rcProcs + 1
				self.TotalStats.rcProcs = self.TotalStats.rcProcs + 1
				if frames.main then
					frames.main.rcActive = true
					frames.main:UpdateInd()
				end
			end
		elseif eventtype == "SPELL_AURA_REMOVED" then
			if param9 == SpellIds["Runic Corruption"] then
				if frames.main then
					frames.main.rcActive = false
					frames.main:UpdateInd()
				end
			end
		end
	end
	if srcGUID == addon.playerGUID then
		if eventtype == "SPELL_ENERGIZE" then
			if param10 == SpellNames["Runic Empowerment"] then
				self.LastFightStats.reProcs = self.LastFightStats.reProcs + 1
				self.TotalStats.reProcs = self.TotalStats.reProcs + 1
			elseif param10 == SpellNames["Pestilent Pustules"] then
				self.LastFightStats.ppProcs = self.LastFightStats.ppProcs + 1
				self.TotalStats.ppProcs = self.TotalStats.ppProcs + 1
				if self.db.profile.debug then
					local fmt = "Pestilent Pustules %s %s %s"
					self:Print(fmt:format(tostring(param11), tostring(param12), tostring(param13)))
				end
			elseif param10 == SpellNames["Murderous Efficiency"] then
				self.LastFightStats.meProcs = self.LastFightStats.meProcs + 1
				self.TotalStats.meProcs = self.TotalStats.meProcs + 1
			end
			if self.db.profile.runeDebug then
				local fmt = "Energize: %s %s %s %s"
				self:Print(fmt:format(tostring(param10), tostring(param11), tostring(param12), tostring(param13)))
			end
		end
	end
	addon:BarDisplayUpdateForEvent("CombatLogEvent", ...)
end
