local _G = getfenv(0)
local ADDON_NAME, addon = ...

local ipairs = _G.ipairs
local pairs = _G.pairs
local floor = _G.math.floor
local select = _G.select
local tostring = _G.tostring

local round = addon.round
local MIN_UPDATE_TIME = addon.MIN_UPDATE_TIME

addon.scripts = {}

local function onTimerExpire(self)
	-- Attempt to invoke a callback on a Bar.
	if not self.parent then return end
	local bar = self.parent

	if bar.OnTimerExpire and _G.type(bar.OnTimerExpire) == "function" then
		bar.OnTimerExpire(bar)
	end
end

addon.scripts.onUpdateRune = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.showing.timer = self.showing.timer + elapsed
	if self.lastUpdate >= MIN_UPDATE_TIME then
		self.lastUpdate = 0
		local profile = addon.db.profile
		self:SetValue(self.showing.timer)

		local remain = self.showing.duration - self.showing.timer
		if remain >= 0 then
			local almostReadyTime = profile.almostReadyTime
			if almostReadyTime > 0 and almostReadyTime >= remain then
				self:SetAlpha(profile.almostReadyAlpha)
			end
			if profile.rechargeTimers ~= "None" then
				if self.showing.start > 0 and
					(self.status ~= "depleted" or profile.rechargeTimers == "All") then
					self.value:SetText(tostring(round(remain)))
				else
					self.value:SetText("")
				end
			end
		else
			self:SetScript("OnUpdate", nil)
		end
	end
end

addon.scripts.onUpdateDisease = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.timer = self.timer - elapsed
	if self.lastUpdate >= MIN_UPDATE_TIME then
		self.lastUpdate = 0
		if self.active then
			if self.timer < 0 then
				self.timer = 0
				self.active = false
			end

			self:Show()
			self:SetValue(self.timer)
			self.value:SetText(tostring(round(self.timer)))
		else
			self.value:SetText("0")
			self:Hide()
			self:SetScript("OnUpdate", nil)
		end
	end
end

addon.scripts.onUpdateTimerWithCounter = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.timer = self.timer - elapsed
	if self.lastUpdate >= MIN_UPDATE_TIME then
		self.lastUpdate = 0
		if self.active then
			if self.timer < 0 then
				self.timer = 0
				self.active = false
				self:SetScript("OnUpdate", nil)
				self:Hide()
			else
				self:Show()
				self:SetValue(self.timer)
				--if self.hasCounter then
				self.value:SetText(tostring(round(self.timer)))
				--end
			end
		else
			self:Hide()
		end
	end
end

addon.scripts.onUpdateTimerWithCounterExpire = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.timer = self.timer - elapsed
	if self.lastUpdate >= MIN_UPDATE_TIME then
		self.lastUpdate = 0
		if self.active then
			if self.timer < 0 then
				self.timer = 0
				self.active = false
				self:SetScript("OnUpdate", nil)
				self:Hide()
				onTimerExpire(self)
			else
				self:Show()
				self:SetValue(self.timer)
				--if self.hasCounter then
				self.value:SetText(tostring(round(self.timer)))
				--end
			end
		else
			self:Hide()
			onTimerExpire(self)
		end
	end
end

addon.scripts.onUpdateTimerWithSecondaryCounter = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.timer = self.timer - elapsed
	if self.lastUpdate >= MIN_UPDATE_TIME then
		self.lastUpdate = 0
		if self.active then
			if self.timer < 0 then
				self.timer = 0
				self.active = false
				self:SetScript("OnUpdate", nil)
				self:Hide()
			else
				self:Show()
				self:SetValue(self.timer)
				--if self.hasCounter then
				self.secondaryValue:SetText(tostring(round(self.timer)))
				--end
			end
		else
			self:Hide()
		end
	end
end

addon.scripts.onUpdateBoneShield = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.timer = self.timer - elapsed
	if self.lastUpdate >= 0.1 then
		if self.active then
			local profile = addon.db.profile.bars["BoneShieldBar"]
			if self.timer > 0 then
				self:Show()
				if profile.progress == "Time" then
					self:SetValue(self.timer)
				elseif profile.progress == "Charges" then
					self:SetValue(self.count)
				end
			else
				self.timer = 0
				self.active = false
				self:SetScript("OnUpdate", nil)
				self:Hide()
			end
		else
			self:Hide()
		end
		self.lastUpdate = 0
	end
end

addon.scripts.onUpdateIconTimer = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.timer = self.timer - elapsed
	if self.lastUpdate >= MIN_UPDATE_TIME then
		self.lastUpdate = 0
		if self.active then
			if self.timer < 0 then
				self.timer = 0
				self.active = false
				self:SetScript("OnUpdate", nil)
				self:SetDesiredAlpha()
			else
				--self:SetValue(self.timer)
				self.value:SetText(tostring(round(self.timer)))
			end
		else
			self:SetDesiredAlpha()
		end
	end
end

addon.scripts.onUpdateTimerNoCounter = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.timer = self.timer - elapsed
	if self.lastUpdate >= MIN_UPDATE_TIME then
		self.lastUpdate = 0
		if self.active then
			if self.timer < 0 then
				self.timer = 0
				self.active = false
				self:SetScript("OnUpdate", nil)
				self:Hide()
			else
				self:Show()
				self:SetValue(self.timer)
			end
		else
			self:Hide()
		end
	end
end

addon.scripts.onUpdateIncreasingTimerNoCounter = function(self, elapsed)
	self.lastUpdate = (self.lastUpdate or 0) + elapsed
	self.timer = self.timer + elapsed
	if self.lastUpdate >= MIN_UPDATE_TIME then
		self.lastUpdate = 0
		if self.active then
			if self.timer > self.duration then
				self.timer = 0
				self.active = false
				self:SetScript("OnUpdate", nil)
				self:Hide()
			else
				self:Show()
				self:SetValue(self.timer)
			end
		else
			self.timer = 0
			self.active = false
			self:Hide()
		end
	end
end
