local _G = getfenv(0)
local ADDON_NAME, addon = ...

local LibStub = _G.LibStub
local CompactRunes = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)
local L = LibStub("AceLocale-3.0"):GetLocale(addon.addonName)

-- Use BfA+ version to search by name.
local UnitBuff = addon.UnitBuff
local UnitDebuff = addon.UnitDebuff

local ipairs = _G.ipairs
local tostring = _G.tostring
local tonumber = _G.tonumber
local GetTime = _G.GetTime
local GetSpellCooldown = _G.GetSpellCooldown
local GetSpellCharges = _G.GetSpellCharges
-- Locals for spell data.
local SpellIds = addon.SpellIds
local SpellNames = addon.SpellNames
local SpellIcons = addon.SpellIcons
-- Local definition of Bar
local Bar = addon.Bar

local frames = addon.frames

function CompactRunes:CreateOtherFrames()
	frames.ffBar = Bar:Create({
		name = "FrostFeverBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-left" },
		functions = {
		}
	})

	frames.bpBar = Bar:Create({
		name = "BloodPlagueBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar1-right" },
		functions = {
		}
	})

	frames.ddBar = Bar:Create({
		name = "DeathAndDecayBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "leftbar-full" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local start, duration, enabled = GetSpellCooldown(SpellIds[self.spellforcd])
					if duration > addon.runeCD then
						self.bar.timer = start + duration - GetTime()
						local bc = self.db.alt_color
						local tc = self.db.alt_textcolor
						self.bar:SetMinMaxValues(0, duration)
						self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
						self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
					else
						local bc = self.db.color
						local tc = self.db.textcolor
						self.bar:SetMinMaxValues(0, 1)
						self.bar:SetValue(1)
						self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
						self.bar.value:SetTextColor(tc.r, tc.g, tc.b, 0)
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", nil)
					end
				end
			end,
			IsEnabled = function(self)
				return self.db.enabled
			end,
			OnTalentUpdate = function(self)
				self.spellforcd = "Death and Decay"
				if self.IsEnabled and self:IsEnabled() and 
					addon.IsSpellKnown(self.spellforcd) then
					self.active = true
					addon:BarDisplayAdd("SpellCooldown", self)
					self:UpdateDisplay()
				else
					self.active = false
					addon:BarDisplayRemove("SpellCooldown", self)
					self:Hide()
				end
			end
		}
	})
	frames.ddBar:OnTalentUpdate()

	frames.rtBar = Bar:Create({
		name = "RuneTapBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "topbar2-full", "topbar2-right" },
		functions = {
			UpdateDisplay = function(self, event)
				if self.active then
					if event == "CombatStart" then
						self.bar:SetAlpha(self.db.alpha)
						return
					end
					local current, max, start, duration = GetSpellCharges(SpellIds["Rune Tap"])
					if current and current >= 0 then
						self.bar.value:SetText(tostring(current))
						self.bar:SetMinMaxValues(0, duration)
						if current >= max then
							self.bar:SetValue(duration)
							self.bar.active = false
							self.bar:SetScript("OnUpdate", nil)
							self:Show()
							self.bar:SetAlpha(_G.UnitAffectingCombat("player") and
								self.db.alpha or addon.db.profile.auxBarOOCAlpha)
						else
							local remaining = GetTime() - start
							self.bar:SetValue(remaining)
							self.bar.active = true
							self.bar.duration = duration
							self.bar.timer = remaining
							self:Show()
							self.bar:SetScript("OnUpdate", addon.scripts.onUpdateIncreasingTimerNoCounter)
							self.bar:SetAlpha(self.db.alpha)
						end
					else
						self:Hide()
						self.bar.active = false
						self.bar:SetScript("OnUpdate", nil)
					end
				else
					self:Hide()
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				local events = { "SpellCooldown", "CombatStart", "CombatEnd" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.rtBar:OnTalentUpdate()

	frames.vbBar = Bar:Create({
		name = "VampiricBloodBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "rightbar-full" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local show = false
					name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
					= UnitBuff("player", SpellNames["Vampiric Blood"])
					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						local bc = self.db.color
						local tc = self.db.textcolor
						self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
						self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
						show = true
					else
						if self.db.showCooldown then
							local start, duration, enabled = GetSpellCooldown(SpellIds["Vampiric Blood"])
							if duration > 2 then
								self.bar.timer = start + duration - GetTime()
								local bc = self.db.alt_color
								local tc = self.db.alt_textcolor
								self.bar:SetMinMaxValues(0, duration)
								self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
								self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
								self.bar.active = true
								self.bar:Show()
								self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
								show = true
							end
						end
					end
					if not show then
						self.bar.active = false
						self.bar.timer = 0
						self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				end
			end,
			IsEnabled = function(self)
				return addon.currentSpec == "Blood" and self.db.enabled
			end,
			OnTalentUpdate = function(self)
				local events = { "SpellCooldown", "PlayerAura" }
				if self.IsEnabled and self:IsEnabled() then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.vbBar:OnTalentUpdate()

	frames.gfBar = Bar:Create({
		name = "GhoulFrenzyBar",
		fontAdj = 2,
		initTimer = true,
		layout = { "rightbar-full" },
		functions = {
			UpdateDisplay = function(self)
				if self.active then
					local show = false
					name, icon, count, dispelType, duration, expires,
					caster, isStealable, shouldConsolidate, spellId, canApplyAura,
					isBossDebuff, castByPlayer, value1, value2, value3
					= UnitBuff("pet", SpellNames[self.spellName])
					if name then
						self.bar.timer = expires - GetTime()
						self.bar:SetMinMaxValues(0, duration)
						local bc = self.db.color
						local tc = self.db.textcolor
						self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
						self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
						self.bar.active = true
						self.bar:Show()
						self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
						show = true
					else
						if self.db.showCooldown then
							local start, duration, enabled = GetSpellCooldown(SpellIds[self.spellName])
							if duration > 2 then
								self.bar.timer = start + duration - GetTime()
								local bc = self.db.alt_color
								local tc = self.db.alt_textcolor
								self.bar:SetMinMaxValues(0, duration)
								self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
								self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
								self.bar.active = true
								self.bar:Show()
								self.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerWithCounter)
								show = true
							end
						end
					end
					if not show then
						self.bar.active = false
						self.bar.timer = 0
						self.bar:SetScript("OnUpdate", nil)
						self.bar:Hide()
					end
				end
			end,
			IsEnabled = function(self)
				return self.db.enabled
			end,
			OnTalentUpdate = function(self)
				self.spellName = "Ghoul Frenzy"
				local events = { "SpellCooldown", "PetAura" }
				if self.IsEnabled and self:IsEnabled() and 
					addon.IsSpellKnown(self.spellName) then
					self.active = true
					for i, event in ipairs(events) do
						addon:BarDisplayAdd(event, self)
					end
					self:UpdateDisplay()
				else
					self.active = false
					for i, event in ipairs(events) do
						addon:BarDisplayRemove(event, self)
					end
					self:Hide()
				end
			end
		}
	})
	frames.gfBar:OnTalentUpdate()

end
