local _G = getfenv(0)
local ADDON_NAME, addon = ...

local CompactRunes = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)
local L = _G.LibStub("AceLocale-3.0"):GetLocale(addon.addonName, true)

local ipairs = _G.ipairs
local pairs = _G.pairs
local UnitBuff = addon.UnitBuff
local UnitDebuff = addon.UnitDebuff
local GetSpellCooldown = _G.GetSpellCooldown
local SpellIds = addon.SpellIds
local SpellNames = addon.SpellNames
local frames = addon.frames

function addon.SetupSpellAuras()
	addon.StandardDiseases = {
		["Any"] = {
			[SpellNames["Blood Plague"]] = frames.bpBar,
			[SpellNames["Frost Fever"]] = frames.ffBar,
		}
	}
	addon.petBuffs = {
	}
end

function addon.GetRuneOrderMap()
	return {
		["BFU"] = L["Blood, Frost, Unholy"],
		["BUF"] = L["Blood, Unholy, Frost"],
		["FBU"] = L["Frost, Blood, Unholy"],
		["UFB"] = L["Unholy, Frost, Blood"],
	}
end

function addon.NeedsGhoul()
	-- Check for Master of Ghouls
	local _, _, _, _, rank = GetTalentInfo(3,6)
	if rank == 0 then return false end

	local start, duration, enabled = GetSpellCooldown(SpellIds["Raise Dead"])
	return duration <= 0
end

function CompactRunes:CheckSpells()
end
  
-- If procs settings changed, update any necessary settings.
function CompactRunes:OnProcsChanged()
end

local presences = {
	"Blood Presence",
	"Frost Presence",
	"Unholy Presence"
}
function CompactRunes:CheckPresence()
	local missingPresence = 1

	for i, presence in ipairs(presences) do
		name, icon, count, dispelType, duration, expires,
		caster, isStealable, shouldConsolidate, spellId, canApplyAura,
		isBossDebuff, castByPlayer, value1, value2, value3
		= UnitBuff("player", SpellNames[presence], "PLAYER")
		if name then missingPresence = nil end
	end

	if self.currentProcs["Missing Presence"] ~= missingPresence then
		self.currentProcs["Missing Presence"] = missingPresence
		self:UpdateProcIcon()
	end
end

function CompactRunes:OnUnitAura(event, unit)
	if unit == "player" then
		self:CheckPresence()
	end
end

function addon.GetActiveTalentSpec()
	-- GetActiveTalentGroup()  returns 1 or 2
	return _G.GetActiveTalentGroup()
end

function CompactRunes:ACTIVE_TALENT_GROUP_CHANGED()
end

function CompactRunes:SPELL_UPDATE_USABLE()
	for k, v in pairs(self.UsableProcs) do
		if k and v then
			local id = SpellIds[k]
			local usable, nomana =_G.IsUsableSpell(id)
			local newValue = usable and 1 or nil
			if self.currentProcs[k] ~= newValue then
				changed = true
				self.currentProcs[k] = newValue
			end
		end
	end

	if changed then
		self:UpdateProcIcon()
	end
end

function CompactRunes:COMBAT_LOG_EVENT_UNFILTERED(...)
end

function CompactRunes:GetMeleeRangeSpells()
	return {
		"Blood Strike",
	}
end

function CompactRunes:OnCheckTalents()
	addon:UpdateSpellsWithRanks()
end

function addon:UpdateSpellsWithRanks()
	for spell, ids in pairs(addon.SpellsWithRanks) do
		if spell and ids and _G.type(ids) == "table" then
			-- Iterate in reverse order, stop on first known
			for i = #ids, 1, -1 do
				local id = ids[i]
				if id and _G.type(id) == "number" then
					if _G.IsSpellKnown(id) then
						addon.SpellIds[spell] = id
						addon:UpdateSpellInfo(spell, id)
						if addon.db.profile.debug then
							local fmt = "Updating spell %s to %s"
							addon:Print(fmt:format(_G.tostring(spell), _G.tostring(id)))
						end
						break
					end
				end
			end
		end
	end
end